function [ ll ] = MillerLFRMLogLikelihood( graph, Z, W, eps, isSymmetric,debug)
%Log-likelihood function for the Miller et al. nonparametric (IBP) link
%prediction model.
    ll = 0;
    numActors = size(Z,1);
    
    matrix = zeros(numActors,numActors);
    
    if nargin < 5
        isSymmetric = true; %by default assume we are interested in symmetric relations (corresponding to a symmetric graph)
    end
    
    if nargin < 6
        debug = false;
    end
    
    for i=1:numActors
        if isSymmetric
            jStart = i + 1;
        else
            jStart = 1;
        end
        for j=jStart:numActors
            if i == j
                continue;
            end
            if graph(i,j) == 1
                %ll = ll - log(1 + exp(-(Z(i,:) * W * Z(j,:)' + eps)));
                ll = ll + log(1./(1 + exp(-(Z(i,:) * W * Z(j,:)' + eps))));
                matrix(i,j) = log(1./(1 + exp(-(Z(i,:) * W * Z(j,:)' + eps))));
            else
                ll = ll + log(1 - 1./(1 + exp(-(Z(i,:) * W * Z(j,:)' + eps))));
                matrix(i,j) = log(1 - 1./(1 + exp(-(Z(i,:) * W * Z(j,:)' + eps))));
            end
        end
    end
    
    
        
    if (nargin==6 && debug==1)
        matrix
        ll
        pause;
    end
    
end