function [ ll ] = MillerLFRMLogLikelihoodFast( graph, logSZWZ, logOneMinusSZWZ, numGraphs, isSymmetric)
% AA: Fast Log-likelihood function for the Miller et al. nonparametric (IBP) link
%prediction model.
% 
% graph = the aggregated count matrix representing the sum of individual graphs
% SZWZ = the cached sigmoid(ZWZ' + eps) matrix
% logSZWZ = the cached log(sigmoid(ZWZ' + eps)) matrix
% logOneMinusSZWZ = the cached log(1-sigmoid(ZWZ' + eps)) matrix
% numGraphs = # of graphs


    if nargin < 5
        isSymmetric = true; %by default assume we are interested in symmetric relations (corresponding to a symmetric graph)
    end
    
    %logSZWZ = log(SZWZ);
    %logOneMinusSZWZ = log(1 - SZWZ);
    
    % Replace for-loops with this computation:
    %llMatrix = graph .* logSZWZ + (numGraphs - graph) .* logOneMinusSZWZ;
    
    %llMatrix = graph .* logSZWZ + (numGraphs - graph) .* logOneMinusSZWZ;
    
    
    % AA: We simply need to calculate each entry in this log-likelihood matrix. 
    % It is significantly faster due to the removal of for-loops.  To
    % convince yourself this is correct, try thinking first about the case
    % where there is only 1 graph per time slice (and so "graph" and
    % "numGraphs-graph" are just binary matrices).
    llMatrix = graph .* logSZWZ + (numGraphs - graph) .* logOneMinusSZWZ;

    
    
    if (isSymmetric)
        % To get log-likelihood, just sum up the upper-triangular part of
        % the "llMatrix"
        ll = sum(sum(triu(llMatrix,1)));
    else
        % Remove diagonal
        llMatrix = llMatrix .* (1-eye(length(llMatrix)));
        ll = sum(sum(llMatrix));
    end
    
    
end