%Demo script for DRIFT
%Runs a simple experiment on a small synthetic dataset
%The task is to predict future graphs based on training graphs.

%Parameters for the synthetic data
numActors = 10;
T = 100;
numFeatures = 5;
b = ones(numFeatures,1) .* 0.98;
a = 1-b;
eps = -2;
WPriorSigma = 3;

%MCMC settings
alpha = 1;
gamma = 10;
delta = 1;
defaultPriorSigma = 1;
numSamplesPerChain = 10;
numChains = 5;
burnIn = 50;
sampleGap = 1;
samplesPerSample = 10;
numPredictionTimesteps = 1;

%Generate synthetic dataset
Z = sampleFromIFMM_Prior(numActors, T + numPredictionTimesteps, alpha, gamma, delta, numFeatures, a, b);

%W here is hard-coded to make the Y's more or less deterministic based on the Z's
%and to make the Z's recoverable (if the number of features is fixed)
%W = [0 5 -3;
%     5 0  0;
%    -3 0  2]; %.* 10;
W = random('norm', 0, WPriorSigma,numFeatures,numFeatures);

Z
W

numGraphs = 1; %the number of graphs generated per timestep (the code can handle multiple graphs)
Y = cell(1,numGraphs);
for i = 1:numGraphs
    Y{i} = generate_DRIFT(Z,W, eps);
end

%Separate training and test
Z_future = Z(T+1:T+numPredictionTimesteps,:,:);
Y_future = cell(1,numGraphs);
for i =1:numGraphs
    Y_future{i} = cell(1,numPredictionTimesteps);
    for t=1:numPredictionTimesteps
       Y_future{i}{t} = Y{i}{T+t};
    end
end

Z = Z(1:T,:,:); %remove test Z's from the training set
Y_train = cell(numGraphs); %remove the test graphs from the training set
for i=1:numGraphs
    Y_train{i} = Y{i};
    Y_train{i}(T+1:end) = [];
end
Y = Y_train;
Y_flat = flattenY_ToMiller(Y); %flatten into one timestep for the Miller et al. LFRM model


fprintf('Running LFRM MCMC\n');
MILLER_LFRM_model_samples = DRIFT_MCMC(Y_flat, [], numSamplesPerChain, burnIn, sampleGap,numChains);
fprintf('evaluating future timesteps\n');
LFRM_sampleLogProbs = zeros(length(MILLER_LFRM_model_samples), 1);
for s = 1:length(MILLER_LFRM_model_samples)
    for g = 1:length(Y_future)
        for t=1:numPredictionTimesteps
            LFRM_sampleLogProbs(s) = LFRM_sampleLogProbs(s) + MillerLFRMLogLikelihood(Y_future{g}{t}, squeeze(MILLER_LFRM_model_samples{s}.Z(1,:,:)), MILLER_LFRM_model_samples{s}.W, MILLER_LFRM_model_samples{s}.eps, true);
        end
    end
end
LFRM_ll = logOfSum(LFRM_sampleLogProbs) - log(length(MILLER_LFRM_model_samples));
fprintf('done evaluating future timesteps\n');

fprintf('Running DRIFT MCMC\n');
model_samples = DRIFT_MCMC(Y, [], numSamplesPerChain, burnIn, sampleGap,numChains);
fprintf('evaluating future timesteps\n');
DRIFT_ll = logLikeForFutureTimesteps(model_samples, defaultPriorSigma, samplesPerSample, Y_future, false);
fprintf('done evaluating future timesteps\n');

fprintf('Evaluating using the true Z\n');
Z_testOracle_ll = 0;
for g = 1:length(Y_future)
    for t=1:numPredictionTimesteps
        Z_testOracle_ll = Z_testOracle_ll + MillerLFRMLogLikelihood(Y_future{g}{t}, squeeze(Z_future(t,:,:)), W, eps, true);
    end
end

fprintf('Evaluating using the true last Z from the training set, and the actual model parameters\n');
Z_trainOracleSample.a = a;
Z_trainOracleSample.b = b;
Z_trainOracleSample.W = W; 
Z_trainOracleSample.Z = Z;
Z_trainOracleSample.eps = eps;
Z_trainOracleSamples = cell(1,1);
Z_trainOracleSamples{1,1} = Z_trainOracleSample;
Z_trainOracle_ll = logLikeForFutureTimesteps(Z_trainOracleSamples, defaultPriorSigma, samplesPerSample, Y_future, false);


fprintf('\n\nLog likelihood for the next timestep:\n');
fprintf('----------------------------------------------\n');
fprintf('LFRM: %d\n', LFRM_ll);
fprintf('DRIFT: %d\n', DRIFT_ll);
fprintf('True Z_test Oracle: %d\n', Z_testOracle_ll);
fprintf('True Z_train Oracle: %d\n', Z_trainOracle_ll);
