function [ sample ] = forwardBackwardV2( Y, sample, i, k )
%FWDBTRCK Runs a forward backtrack sweep on a binary markov chain.
%
% sample = forwardBackwardSample(Y, sample, i, k) runs the forward-backtrack sampling
% algorithm on the chain associated with actor i and feature k with all other chains fixed.
% (block Gibbs sampling update)
%Jimmy's implementation

T = length(Y{1});

% Compute the transition matrix parameters for chain k.
a = sample.a(k);
b = sample.b(k);
log_q = [log(1-a), log(a); log(1-b), log(b)]; %Indices are +1 since one based


P = zeros(T,2,2); %P matrices, in logspace
%forward recursion creates the P matrices

sample.Z(1,i,k) = 0;
logPi_t_minusOne(0 + 1) = log(1-a) + computeMultiGraphLogLike(Y,1,sample);
sample.Z(1,i,k) = 1;
logPi_t_minusOne(1 + 1) = log(a) + computeMultiGraphLogLike(Y,1,sample);
for t = 2:T
   for r=0:1
       for s = 0:1
           sample.Z(t,i,k) = s;
           log_P_s = computeMultiGraphLogLike(Y,t,sample);
           
           P(t,r+1,s+1) = logPi_t_minusOne(r + 1) + log_q(r + 1, s + 1) + log_P_s; %currently this is unnormalized and in log space
       end
   end
   P(t,:,:) = normalizeLogProbs(squeeze(P(t,:,:))); %get normalized probabilities back. %TODO can I avoid this squeeze?
   logPi_t_minusOne(0 + 1) = log(sum(sum(P(t,:,0+1))));
   logPi_t_minusOne(1 + 1) = log(sum(sum(P(t,:,1+1))));
end
if (isfinite(sum(logPi_t_minusOne))==0)
    error();
end
%fprintf('forward matrices:\n');
% for t=1:T
%    squeeze(P(t,:,:)) 
% end

%backwards recursion to do the sampling
Pi_T = normalizeLogProbs(logPi_t_minusOne);
sample.Z(T,i,k) = rand() < Pi_T(1 + 1);
if T == 1
    %fprintf('returning already\n');
    return
end
t = T-1;
while t >= 1
    %fprintf('backwards recusion t = %d\n', t);
    PrOne = P(t+1,1+1,sample.Z(t+1,i,k) + 1);
    PrZero = P(t+1,0+1,sample.Z(t+1,i,k) + 1);
    PrOne = PrOne ./ (PrOne + PrZero);
    sample.Z(t,i,k) = rand() < PrOne;
    t = t -1;
end
end
        
function ll = computeMultiGraphLogLike(Y,t,sample)
%compute the Miller log likelihood for all of the graphs in Y at time t for
%the parameters in sample.
numGraphs = size(Y,1);
ll = 0;
for g = 1:numGraphs
    ll = ll + MillerLFRMLogLikelihood(Y{g}{t}, squeeze(sample.Z(t,:,:)), sample.W, sample.eps, true);
end
end

%Takes log probabilities as input, whose probabilities are assumed to sum
%to one, and computes the normalized probability of temp1.
function pr = normalizeLogProbs(logMatrix)
    a = max(max(logMatrix));
    b = exp(logMatrix - a);
    total = sum(sum(b));
    pr = b ./ total;
end