function [ sample ] = forwardBackwardV2Fast( Y, Ydata, sample, i, k )
%FWDBTRCK Runs a forward backtrack sweep on a binary markov chain.
%
% sample = forwardBackwardSample(Y, Ydata, sample, i, k) runs the forward-backtrack sampling
% algorithm on the chain associated with actor i and feature k with all other chains fixed.
% (block Gibbs sampling update)
%Jimmy's implementation
%
% AA: Made significantly faster

    T = length(Y{1});

    % Compute the transition matrix parameters for chain k.
    a = sample.a(k);
    b = sample.b(k);
    log_q = [log(1-a), log(a); log(1-b), log(b)]; %Indices are +1 since one based


    P = zeros(T,2,2); %P matrices, in logspace
    %forward recursion creates the P matrices

    % AA: I order the calculation of the loglikelihood based on the current
    % value of sample.Z(1,i,k) -- so that we can avoid one "updateCache"
    if (sample.Z(1,i,k) == 0)
        % AA: Here I use a new function "computeMultiGraphLogLikeFast" --
        % otherwise its the same command
        logPi_t_minusOne(0 + 1) = log(1-a) + computeMultiGraphLogLikeFast(Y,Ydata,1,sample);

        % AA: Now we have to pretend that Z(1,i,k) has changed to 1 to get the
        % new log-likelihood
        sample.Z(1,i,k) = 1;
        % AA: This updates the cache matrices given that Z(1,i,k) has changed from 0 to 1 
        sample = updateCache(sample,1,i,k,1);
        logPi_t_minusOne(1 + 1) = log(a) + computeMultiGraphLogLikeFast(Y,Ydata,1,sample);    
    else
        % AA: Same thing as above, but now we start by evaluating the loglikelihood
        % when Z(1,i,k) =1
        logPi_t_minusOne(1 + 1) = log(a) + computeMultiGraphLogLikeFast(Y,Ydata,1,sample);

        sample.Z(1,i,k) = 0;
        sample = updateCache(sample,1,i,k,0);
        logPi_t_minusOne(0 + 1) = log(1-a) + computeMultiGraphLogLikeFast(Y,Ydata,1,sample);

    end


    for t = 2:T
       for r=0:1      
           % AA: I'm unrolling the loop to avoid one updateCache computation.  I'm
           % doing the same thing as in time slice 1 above
           if (sample.Z(t,i,k) == 0)
               s=0;
               log_P_s = computeMultiGraphLogLikeFast(Y,Ydata,t,sample); % AA: Using fast function    

               P(t,r+1,s+1) = logPi_t_minusOne(r + 1) + log_q(r + 1, s + 1) + log_P_s; %currently this is unnormalized and in log space

               s=1;
               sample.Z(t,i,k) = s;
               sample = updateCache(sample,t,i,k,s);
               log_P_s = computeMultiGraphLogLikeFast(Y,Ydata,t,sample);   

               P(t,r+1,s+1) = logPi_t_minusOne(r + 1) + log_q(r + 1, s + 1) + log_P_s; %currently this is unnormalized and in log space
           else
               s=1;
               log_P_s = computeMultiGraphLogLikeFast(Y,Ydata,t,sample);
               P(t,r+1,s+1) = logPi_t_minusOne(r + 1) + log_q(r + 1, s + 1) + log_P_s; %currently this is unnormalized and in log space

               s=0;
               sample.Z(t,i,k) = s;
               sample = updateCache(sample,t,i,k,s);
               log_P_s = computeMultiGraphLogLikeFast(Y,Ydata,t,sample);

               P(t,r+1,s+1) = logPi_t_minusOne(r + 1) + log_q(r + 1, s + 1) + log_P_s; %currently this is unnormalized and in log space
           end


       end

       temp2=squeeze(P(t,:,:));
       P(t,:,:) = normalizeLogProbs(P(t,:,:)); %get normalized probabilities back  

       logPi_t_minusTwo = logPi_t_minusOne;

       if (nnz(P(t,:,:)) < 4)  %hack to avoid numerical problems!
           P(t,:,:) = P(t,:,:) + 0.0001;
           P(t,:,:) = P(t,:,:) ./ sum(sum(P(t,:,:)));
           squeeze(P(t,:,:))
       end

       logPi_t_minusOne(0 + 1) = log(sum(sum(P(t,:,0+1))));
       logPi_t_minusOne(1 + 1) = log(sum(sum(P(t,:,1+1))));

       if (isfinite(sum(logPi_t_minusOne))==0)

           logPi_t_minusTwo
           log_q
           a
           b
           temp2
           logPi_t_minusOne
           log_P_s
           squeeze(P(t,:,:))
           sample.W
           squeeze(sample.Z(t,:,:))
           squeeze(P(t-1,:,:))
           error('error!')
       end
    end
    %fprintf('forward matrices:\n');
    % for t=1:T
    %    squeeze(P(t,:,:)) 
    % end

    %backwards recursion to do the sampling
    Pi_T = normalizeLogProbs(logPi_t_minusOne);

    previousZ = sample.Z(T,i,k);
    sample.Z(T,i,k) = rand() < Pi_T(1 + 1);

    if (sample.Z(T,i,k) - previousZ ~= 0) % AA: if Z was changed, need to update cache
        sample = updateCache(sample,T,i,k,sample.Z(T,i,k));
    end



    if T == 1
        %fprintf('returning already\n');
        return
    end
    t = T-1;
    while t >= 1
        %fprintf('backwards recusion t = %d\n', t);
        PrOne = P(t+1,1+1,sample.Z(t+1,i,k) + 1);
        PrZero = P(t+1,0+1,sample.Z(t+1,i,k) + 1);
        PrOne = PrOne ./ (PrOne + PrZero);

        previousZ = sample.Z(t,i,k);
        sample.Z(t,i,k) = rand() < PrOne;

        if (sample.Z(t,i,k) - previousZ ~= 0) % if different, need to update cache
            sample = updateCache(sample,t,i,k,sample.Z(t,i,k));
        end

        t = t -1;
    end
end
        
% AA: This is a new helper function that updates the cache matrices.
function [sample] = updateCache(sample,t,actor,feature,onFlag)
% Updates the cache statistics, given that Z(t,actor,feature) was changed
% to be 1 or 0 (depending on "onFlag")

    if (onFlag) % if Z(t,actor,feature) changed from 0->1
        % Only need to modify one column of WZ (by adding a column of W, since
        % now there is a 1 in Z(t,actor,feature).
        sample.cache(t).wz(:,actor) = sample.cache(t).wz(:,actor) + sample.W(:,feature);
        % Have to modify one row and column of ZWZ
        sample.cache(t).zwz(actor,:) = sample.cache(t).zwz(actor,:) + sample.cache(t).wz(feature,:);
        sample.cache(t).zwz(:,actor) = sample.cache(t).zwz(actor,:)'; % Assuming symmetric W
    else % if Z(t,actor,feature) changed from 1->0
        sample.cache(t).wz(:,actor) = sample.cache(t).wz(:,actor) - sample.W(:,feature);
        sample.cache(t).zwz(actor,:) = sample.cache(t).zwz(actor,:) - sample.cache(t).wz(feature,:);
        sample.cache(t).zwz(:,actor) = sample.cache(t).zwz(actor,:)'; % Assuming symmetric W
    end

    sample.cache(t).zwz(actor,actor) = 0; % diagonal entry not needed if grouping is off

    % Now need to take sigmoid of changed entries
    sample.cache(t).szwz(actor,:) = 1 ./ (1 + exp(-(sample.cache(t).zwz(actor,:) + sample.eps)));
    sample.cache(t).szwz(:,actor) = sample.cache(t).szwz(actor,:)';  % Assuming symmetric W
    % Now calculate log(sigmoid) and log(1-sigomoid) of changed entries
    sample.cache(t).logSZWZ(actor,:) = log(sample.cache(t).szwz(actor,:));
    sample.cache(t).logSZWZ(:,actor) = sample.cache(t).logSZWZ(actor,:)';
    sample.cache(t).logOneMinusSZWZ(actor,:) = log(1-sample.cache(t).szwz(actor,:));
    sample.cache(t).logOneMinusSZWZ(:,actor) = sample.cache(t).logOneMinusSZWZ(actor,:)';

end

function ll = computeMultiGraphLogLike(Y,t,sample)
%compute the Miller log likelihood for all of the graphs in Y at time t for
%the parameters in sample.
%This is the non cache-accelerated version, obsolete but useful for debug.
    numGraphs = size(Y,1);
    ll = 0;
    for g = 1:numGraphs
        ll = ll + MillerLFRMLogLikelihood(Y{g}{t}, squeeze(sample.Z(t,:,:)), sample.W, sample.eps, true);
    end
end

%Takes log probabilities as input, whose probabilities are assumed to sum
%to one, and computes the normalized probability of temp1.
function pr = normalizeLogProbs(logMatrix)
    a = max(max(logMatrix));
    %a = mean(mean(logMatrix));
    b = exp(logMatrix - a);
    total = sum(sum(b));
    pr = b ./ total;
end