function [ Y ] = generate_DRIFT(Z,W, eps)
%Simulate from the network portion of DRIFT.
%Takes as parameters the Z, W and epsilon values.
%To sample these as well, the code in the initialization function of
%DRIFT_MCMC can be used, eg:
%sample.W = random('norm', 0, WPriorSigma,numFeatures,numFeatures)
    if nargin < 3
        eps = 0;
    end
    T = size(Z,1);
    numActors = size(Z,2);
    numFeatures = size(Z,3);
    Y = cell(T,1);
    for t=1:T
        Y_t = zeros(numActors,numActors);
        for i = 1:numActors
           for j = i+1:numActors %I am assuming a symmetric relation
               Y_t(i,j) = rand() < 1./(1 + exp(-(squeeze(Z(t,i,:))' * W * squeeze(Z(t,j,:)) + eps)));
               Y_t(j,i) = Y_t(i,j);
           end
        end
        Y{t} = Y_t;
    end

end

